{: Wake test
}
unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  GLScene, GLTerrainRenderer, GLObjects, GLMisc, jpeg, GLHeightData,
  ExtCtrls, GLCadencer, StdCtrls, GLTexture, GLHUDObjects,
  GLSkydome, GLWin32Viewer, Geometry, GLHeightTileFileHDS, GLWindowsFont,
  GLBitmapFont, ComCtrls;

type
  TForm1 = class(TForm)
    GLSceneViewer1: TGLSceneViewer;
    GLScene1: TGLScene;
    GLCamera1: TGLCamera;
    DummyCube1: TGLDummyCube;
    TerrainRenderer1: TGLTerrainRenderer;
    Timer1: TTimer;
    GLCadencer1: TGLCadencer;
    GLMaterialLibrary1: TGLMaterialLibrary;
    HUDText1: TGLHUDText;
    SkyDome1: TGLSkyDome;
    GLDummyCube1: TGLDummyCube;
    GLHeightTileFileHDS1: TGLHeightTileFileHDS;
    GLWindowsBitmapFont1: TGLWindowsBitmapFont;
    GLCustomHDS1: TGLCustomHDS;
    PAProgress: TPanel;
    ProgressBar: TProgressBar;
    Label1: TLabel;
    procedure GLSceneViewer1MouseDown(Sender: TObject;
      Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure GLSceneViewer1MouseMove(Sender: TObject; Shift: TShiftState;
      X, Y: Integer);
    procedure Timer1Timer(Sender: TObject);
    procedure GLCadencer1Progress(Sender: TObject; const deltaTime,
      newTime: Double);
    procedure FormCreate(Sender: TObject);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
    procedure GLCustomHDS1MarkDirtyEvent(const area: TRect);
    procedure GLCustomHDS1StartPreparingData(heightData: THeightData);
    procedure GLSceneViewer1BeforeRender(Sender: TObject);
  private
    { Dclarations prives }
  public
    { Dclarations publiques }
    mx, my : Integer;
    fullScreen : Boolean;
    FCamHeight : Single;
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

uses Keyboard, OpenGL12;

procedure TForm1.FormCreate(Sender: TObject);
var
   i, j : Integer;
   name : String;
   libMat : TGLLibMaterial;
begin
   SetCurrentDir(ExtractFilePath(Application.ExeName));
   // apply texture map scale (our heightmap size is 256)
//   TerrainRenderer1.TilesPerTexture:=1;//256/TerrainRenderer1.TileSize;
   // load texmaps
   for i:=0 to 3 do for j:=0 to 3 do begin
      name:=format('Tx0%dx0%d.bmp', [i, j]);
      libMat:=GLMaterialLibrary1.AddTextureMaterial(name, name);
      with libMat.Material.Texture do begin
         TextureMode:=tmReplace;
         TextureWrap:=twNone;
         Compression:=tcStandard;
         FilteringQuality:=tfAnisotropic;
      end;
      libMat.Texture2Name:='detail';
   end;
   with GLMaterialLibrary1.Materials[0].Material do begin
      Texture.Image.LoadFromFile('detail.bmp');
   end;
   // Could've been done at design time, but it the, it hurts the eyes ;)
   GLSceneViewer1.Buffer.BackgroundColor:=clWhite;
   // Move camera starting point to an interesting hand-picked location
   DummyCube1.Position.X:=-256*TerrainRenderer1.Scale.X;
   DummyCube1.Position.Z:=256*TerrainRenderer1.Scale.y;
   DummyCube1.Turn(180);
   // Initial camera height offset (controled with pageUp/pageDown)
   FCamHeight:=10;
end;

procedure TForm1.GLCadencer1Progress(Sender: TObject; const deltaTime,
  newTime: Double);
var
   speed : Single;
begin
   // handle keypresses
   if IsKeyDown(VK_SHIFT) then
      speed:=5*deltaTime
   else speed:=deltaTime;
   with GLCamera1.Position do begin
      if IsKeyDown(VK_UP) then
         DummyCube1.Translate(X*speed, 0, Z*speed);
      if IsKeyDown(VK_DOWN) then
         DummyCube1.Translate(-X*speed, 0, -Z*speed);
      if IsKeyDown(VK_LEFT) then
         DummyCube1.Translate(Z*speed, 0, -X*speed);
      if IsKeyDown(VK_RIGHT) then
         DummyCube1.Translate(-Z*speed, 0, X*speed);
      if IsKeyDown(VK_PRIOR) then
         FCamHeight:=FCamHeight+10*speed;
      if IsKeyDown(VK_NEXT) then
         FCamHeight:=FCamHeight-10*speed;
      if IsKeyDown(VK_ESCAPE) then Close;
   end;
   // don't drop through terrain!
   with DummyCube1.Position do
      Y:=TerrainRenderer1.InterpolatedHeight(AsVector)+FCamHeight;
end;

// Standard mouse rotation & FPS code below

procedure TForm1.GLSceneViewer1MouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
   mx:=x;
   my:=y;
end;

procedure TForm1.GLSceneViewer1MouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
begin
   if ssLeft in Shift then begin
      GLCamera1.MoveAroundTarget(my-y, mx-x);
      mx:=x;
      my:=y;
   end;
end;

procedure TForm1.Timer1Timer(Sender: TObject);
begin
   HUDText1.Text:=Format('%.1f FPS - %d',
                         [GLSceneViewer1.FramesPerSecond, TerrainRenderer1.LastTriangleCount]);
   GLSceneViewer1.ResetPerformanceMonitor;
end;

procedure TForm1.FormKeyPress(Sender: TObject; var Key: Char);
var
   i : Integer;
begin
   case Key of
      'w', 'W' : for i:=0 to GLMaterialLibrary1.Materials.Count-1 do begin
         with GLMaterialLibrary1.Materials[i].Material.FrontProperties do begin
            if PolygonMode=pmLines then
               PolygonMode:=pmFill
            else PolygonMode:=pmLines;
         end;
      end;
      '+' : if GLCamera1.DepthOfView<2000 then begin
         GLCamera1.DepthOfView:=GLCamera1.DepthOfView*1.2;
         with GLSceneViewer1.Buffer.FogEnvironment do begin
            FogEnd:=FogEnd*1.2;
            FogStart:=FogStart*1.2;
         end;
      end;
      '-' : if GLCamera1.DepthOfView>300 then begin
         GLCamera1.DepthOfView:=GLCamera1.DepthOfView/1.2;
         with GLSceneViewer1.Buffer.FogEnvironment do begin
            FogEnd:=FogEnd/1.2;
            FogStart:=FogStart/1.2;
         end;
      end;
      '*' : with TerrainRenderer1 do
         if CLODPrecision>1 then CLODPrecision:=Round(CLODPrecision*0.8);
      '/' : with TerrainRenderer1 do
         if CLODPrecision<1000 then CLODPrecision:=Round(CLODPrecision*1.2);
      '8' : with TerrainRenderer1 do
         if QualityDistance>40 then QualityDistance:=Round(QualityDistance*0.8);
      '9' : with TerrainRenderer1 do
         if QualityDistance<1000 then QualityDistance:=Round(QualityDistance*1.2);
      'n', 'N' : with SkyDome1 do if Stars.Count=0 then begin
         // turn on 'night' mode
         Bands[1].StopColor.AsWinColor:=RGB(0, 0, 16);
         Bands[1].StartColor.AsWinColor:=RGB(0, 0, 8);
         Bands[0].StopColor.AsWinColor:=RGB(0, 0, 8);
         Bands[0].StartColor.AsWinColor:=RGB(0, 0, 0);
         with Stars do begin
            AddRandomStars(700, clWhite, True);   // many white stars
            AddRandomStars(100, RGB(255, 200, 200), True);  // some redish ones
            AddRandomStars(100, RGB(200, 200, 255), True);  // some blueish ones
            AddRandomStars(100, RGB(255, 255, 200), True);  // some yellowish ones
         end;
         GLSceneViewer1.Buffer.BackgroundColor:=clBlack;
         with GLSceneViewer1.Buffer.FogEnvironment do begin
            FogColor.AsWinColor:=clBlack;
            FogStart:=-FogStart; // Fog is used to make things darker
         end;
      end;
      'd', 'D' : with SkyDome1 do if Stars.Count>0 then begin
         // turn on 'day' mode
         Bands[1].StopColor.Color:=clrNavy;
         Bands[1].StartColor.Color:=clrBlue;
         Bands[0].StopColor.Color:=clrBlue;
         Bands[0].StartColor.Color:=clrWhite;
         Stars.Clear;
         GLSceneViewer1.Buffer.BackgroundColor:=clWhite;
         with GLSceneViewer1.Buffer.FogEnvironment do begin
            FogColor.AsWinColor:=clWhite;
            FogStart:=-FogStart;
         end;
         GLSceneViewer1.Buffer.FogEnvironment.FogStart:=0;
      end;
      't' : with SkyDome1 do begin
         if sdoTwinkle in Options then
            Options:=Options-[sdoTwinkle]
         else Options:=Options+[sdoTwinkle];
      end;
   end;
   Key:=#0;
end;

procedure TForm1.GLCustomHDS1MarkDirtyEvent(const area: TRect);
begin
   GLHeightTileFileHDS1.MarkDirty(area);
end;

procedure TForm1.GLCustomHDS1StartPreparingData(heightData: THeightData);
var
   htfHD : THeightData;
   i, j : Integer;
begin
   with heightData do
      htfHD:=GLHeightTileFileHDS1.GetData(XLeft, YTop, Size, DataType);
   if htfHD.DataState=hdsNone then
      heightData.DataState:=hdsNone
   else begin
      i:=(heightData.XLeft div 64)-2;
      j:=(heightData.YTop div 64)-2;
      if (Cardinal(i)<4) and (Cardinal(j)<4) then begin
         heightData.MaterialName:=format('Tx0%dx0%d.bmp', [i, j]);
         heightData.TextureCoordinatesMode:=tcmLocal;
         Assert(heightData.DataType=hdtSmallInt);
         htfHD.DataType:=hdtSmallInt;
         heightData.Allocate(hdtSmallInt);
         Move(htfHD.SmallIntData^, heightData.SmallIntData^, htfHD.DataSize);
         heightData.DataState:=hdsReady;
      end else heightData.DataState:=hdsNone
   end;
   GLHeightTileFileHDS1.Release(htfHD);
end;

procedure TForm1.GLSceneViewer1BeforeRender(Sender: TObject);
var
   i, n : Integer;
begin
   PAProgress.Left:=(Width-PAProgress.Width) div 2;
   PAProgress.Visible:=True;
   n:=GLMaterialLibrary1.Materials.Count;
   ProgressBar.Max:=n-1;
   for i:=0 to n-1 do begin
      ProgressBar.Position:=i;
      GLMaterialLibrary1.Materials[i].Material.Texture.Handle;
      PAProgress.Repaint;
   end;
   PAProgress.Visible:=False;
   GLSceneViewer1.BeforeRender:=nil;
end;

end.
